@@ -17,12 +17,15 @@ pub struct PreviousInfo {
 #[derive(Debug)]
 pub struct Data<'a> {
     pub flows: &'a HashMap<String, Flow>,
+    pub extern_flows: &'a HashMap<String, Flow>,
     pub flow: &'a Flow,
+    pub constants: HashMap<String, Literal>,
+    pub default_flow: String,
     pub context: &'a mut Context,
     pub event: &'a Event,
     pub env: &'a Literal,
 
-    pub loop_indexs: Vec<usize>,
+    pub loop_indexes: Vec<usize>,
     pub loop_index: usize,
 
     pub step_count: &'a mut i32,
@@ -57,11 +60,13 @@ impl PreviousInfo {
 impl<'a> Data<'a> {
     pub fn new(
         flows: &'a HashMap<String, Flow>,
+        extern_flows: &'a HashMap<String, Flow>,
         flow: &'a Flow,
+        default_flow: String,
         context: &'a mut Context,
         event: &'a Event,
         env: &'a Literal,
-        loop_indexs: Vec<usize>,
+        loop_indexes: Vec<usize>,
         loop_index: usize,
         step_count: &'a mut i32,
         step_vars: HashMap<String, Literal>,
@@ -69,13 +74,18 @@ impl<'a> Data<'a> {
         custom_component: &'a serde_json::Map<String, serde_json::Value>,
         native_component: &'a serde_json::Map<String, serde_json::Value>,
     ) -> Self {
+        let constants = flow.constants.clone();
+
         Self {
             flows,
+            extern_flows,
             flow,
+            constants,
+            default_flow,
             context,
             event,
             env,
-            loop_indexs,
+            loop_indexes,
             loop_index,
             step_count,
             step_vars,
@@ -88,8 +98,10 @@ impl<'a> Data<'a> {
     pub fn copy_scope(
         &self,
     ) -> (
+        HashMap<String, Flow>,
         HashMap<String, Flow>,
         Flow,
+        String,
         Context,
         Event,
         Literal,
@@ -102,11 +114,13 @@ impl<'a> Data<'a> {
     ) {
         (
             self.flows.clone(),
+            self.extern_flows.clone(),
             self.flow.clone(),
+            self.default_flow.to_string(),
             init_child_context(&self),
             self.event.clone(),
             self.env.clone(),
-            self.loop_indexs.clone(),
+            self.loop_indexes.clone(),
             self.loop_index.clone(),
             self.step_count.clone(),
             self.step_vars.clone(),
@@ -142,11 +156,13 @@ pub fn init_child_scope<'a>(
 ) -> Data<'a> {
     Data::new(
         &data.flows,
+        &data.extern_flows,
         &data.flow,
+        data.default_flow.clone(),
         context,
         &data.event,
         &data.env,
-        data.loop_indexs.clone(),
+        data.loop_indexes.clone(),
         data.loop_index,
         step_count,
         HashMap::new(),
