@@ -1,33 +1,33 @@
 use crate::data::error_info::ErrorInfo;
 use crate::data::position::Position;
 use crate::data::{
-    literal,
     ast::Interval,
+    csml_logs::*,
+    literal,
     literal::ContentType,
     message::Message,
-    csml_logs::*,
     primitive::{
         tools_crypto, tools_jwt, tools_smtp, tools_time, Data, MessageData, Primitive,
         PrimitiveArray, PrimitiveBoolean, PrimitiveInt, PrimitiveNull, PrimitiveString,
         PrimitiveType, Right, MSG,
     },
     tokens::TYPES,
-    Literal,
+    Literal, MemoryType,
 };
 use crate::error_format::*;
 use crate::interpreter::{
-    builtins::http::{http_request}, json_to_rust::json_to_literal,
+    builtins::http::http_request, json_to_rust::json_to_literal,
     variable_handler::match_literals::match_obj,
 };
 use std::cmp::Ordering;
 use std::{collections::HashMap, sync::mpsc};
 
+use chrono::{DateTime, FixedOffset, LocalResult, TimeZone, Utc};
+use chrono_tz::{Tz, UTC};
+use lettre::Transport;
 use phf::phf_map;
 use regex::Regex;
-use lettre::Transport;
 use serde::{Deserialize, Serialize};
-use chrono::{DateTime, LocalResult, TimeZone, Utc, FixedOffset};
-use chrono_tz::Tz;
 
 ////////////////////////////////////////////////////////////////////////////////
 // DATA STRUCTURES
@@ -91,6 +91,7 @@ const FUNCTIONS_EVENT: phf::Map<&'static str, (PrimitiveMethod, Right)> = phf_ma
     "get_type" => (PrimitiveObject::get_type as PrimitiveMethod, Right::Read),
     "get_content" => (PrimitiveObject::get_content as PrimitiveMethod, Right::Read),
     "is_email" => (PrimitiveObject::is_email as PrimitiveMethod, Right::Read),
+    "is_secure" => (PrimitiveObject::is_secure as PrimitiveMethod, Right::Read),
     "match" => (PrimitiveObject::match_args as PrimitiveMethod, Right::Read),
     "match_array" => (PrimitiveObject::match_array as PrimitiveMethod, Right::Read),
 };
@@ -102,6 +103,8 @@ const FUNCTIONS_READ: phf::Map<&'static str, (PrimitiveMethod, Right)> = phf_map
     "type_of" => (PrimitiveObject::type_of as PrimitiveMethod, Right::Read),
     "get_info" => (PrimitiveObject::get_info as PrimitiveMethod, Right::Read),
     "is_error" => (PrimitiveObject::is_error as PrimitiveMethod, Right::Read),
+    "to_xml" => (PrimitiveObject::to_xml as PrimitiveMethod, Right::Read),
+    "to_yaml" => (PrimitiveObject::to_yaml as PrimitiveMethod, Right::Read),
     "to_string" => (PrimitiveObject::to_string as PrimitiveMethod, Right::Read),
 
     "contains" => (PrimitiveObject::contains as PrimitiveMethod, Right::Read),
@@ -475,8 +478,7 @@ impl PrimitiveObject {
         }
 
         if let Some(literal) = object.value.get("method") {
-
-            let method =  match Literal::get_value::<String>(
+            let method = match Literal::get_value::<String>(
                 &literal.primitive,
                 &data.context.flow,
                 interval,
@@ -489,14 +491,19 @@ impl PrimitiveObject {
                 Ok(get) if get == "get" => "get",
                 _ => {
                     return Err(gen_error_info(
-                        Position::new(interval, &data.context.flow,),
+                        Position::new(interval, &data.context.flow),
                         ERROR_HTTP_UNKNOWN_METHOD.to_string(),
                     ))
                 }
             };
 
-            let value = http_request(&object.value, method, &data.context.flow, interval, false)?;
-            return json_to_literal(&value, interval, &data.context.flow);
+            let (value, response_info) =
+                http_request(&object.value, method, &data.context.flow, interval, false)?;
+            let mut literal = json_to_literal(&value, interval, &data.context.flow)?;
+            // add additional information about the http request response: status and headers
+            literal.add_info_block(response_info);
+
+            return Ok(literal);
         }
 
         Err(gen_error_info(
@@ -673,7 +680,6 @@ impl PrimitiveObject {
         interval: Interval,
         _content_type: &str,
     ) -> Result<Literal, ErrorInfo> {
-
         let usage = "Available mechanisms: PLAIN, AUTH LOGIN, XOAUTH2. set_auth_mechanism(String || Array<String>) => smtp object";
 
         let auth_mechanisms = match args.get("arg0") {
@@ -681,22 +687,20 @@ impl PrimitiveObject {
                 let value = tools_smtp::get_auth_mechanism(lit, data, interval, usage)?;
 
                 let mut map = HashMap::new();
-                map.insert(
-                    value,
-                    PrimitiveNull::get_literal(interval),
-                );
+                map.insert(value, PrimitiveNull::get_literal(interval));
 
                 map
-            },
+            }
             Some(lit) if lit.content_type == "array" => {
                 let vec = Literal::get_value::<Vec<Literal>>(
-                  &lit.primitive,
+                    &lit.primitive,
                     &data.context.flow,
                     lit.interval,
                     format!("usage: {}", usage),
                 )?;
 
-                let map = vec.iter()
+                let map = vec
+                    .iter()
                     .filter_map(|lit| {
                         tools_smtp::get_auth_mechanism(lit, data, interval, usage).ok()
                     })
@@ -711,16 +715,16 @@ impl PrimitiveObject {
                 }
 
                 map
-            },
+            }
             _ => {
                 csml_logger(
                     CsmlLog::new(
                         None,
                         Some(data.context.flow.to_string()),
                         Some(interval.start_line),
-                        format!("set_auth_mechanism wrong mechanism name {:?}", args)
+                        format!("set_auth_mechanism wrong mechanism name {:?}", args),
                     ),
-                    LogLvl::Error
+                    LogLvl::Error,
                 );
 
                 return Err(gen_error_info(
@@ -827,18 +831,18 @@ impl PrimitiveObject {
                 None,
                 Some(data.context.flow.to_string()),
                 Some(interval.start_line),
-                format!("send email: {:?}", email)
+                format!("send email: {:?}", email),
             ),
-            LogLvl::Info
+            LogLvl::Info,
         );
         csml_logger(
             CsmlLog::new(
                 None,
                 Some(data.context.flow.to_string()),
                 Some(interval.start_line),
-                format!("send email: {:?}, mailer: {:?}", email, object.value)
+                format!("send email: {:?}, mailer: {:?}", email, object.value),
             ),
-            LogLvl::Debug
+            LogLvl::Debug,
         );
         let mailer = tools_smtp::get_mailer(&mut object.value, data, interval)?;
 
@@ -850,14 +854,14 @@ impl PrimitiveObject {
                         None,
                         Some(data.context.flow.to_string()),
                         Some(interval.start_line),
-                        format!("send email failed {:?}", e)
+                        format!("send email failed {:?}", e),
                     ),
-                    LogLvl::Error
+                    LogLvl::Error,
                 );
                 return Err(gen_error_info(
                     Position::new(interval, &data.context.flow),
                     format!("Could not send email: {:?}", e),
-                ))
+                ));
             }
         }
     }
@@ -914,7 +918,7 @@ impl PrimitiveObject {
 
         let timezone = match args.get("arg0") {
             Some(lit) if lit.primitive.get_type() == PrimitiveType::PrimitiveString => {
-                let tz_name =  Literal::get_value::<String>(
+                let tz_name = Literal::get_value::<String>(
                     &lit.primitive,
                     &data.context.flow,
                     interval,
@@ -929,14 +933,15 @@ impl PrimitiveObject {
                         ));
                     }
                 };
-            
 
                 tz.to_string()
             }
-            _ => return Err(gen_error_info(
-                Position::new(interval, &data.context.flow),
-                format!("usage: {}", usage),
-            ))
+            _ => {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    format!("usage: {}", usage),
+                ))
+            }
         };
 
         object.value.insert(
@@ -971,10 +976,10 @@ impl PrimitiveObject {
 
                 match time_value {
                     t_val if t_val == "s" => t_val.to_owned(),
-                    _  => "m".to_owned()
+                    _ => "m".to_owned(),
                 }
             }
-            _ => "m".to_owned()
+            _ => "m".to_owned(),
         };
 
         match object.value.get("milliseconds") {
@@ -990,7 +995,7 @@ impl PrimitiveObject {
 
                 let duration = match time_type {
                     t_val if t_val == "s" => date.timestamp(),
-                    _  => date.timestamp_millis()
+                    _ => date.timestamp_millis(),
                 };
 
                 Ok(PrimitiveInt::get_literal(duration, interval))
@@ -1047,10 +1052,12 @@ impl PrimitiveObject {
 
                 Ok(lit)
             }
-            _ => return Err(gen_error_info(
-                Position::new(interval, &data.context.flow),
-                format!("usage: {}", usage),
-            ))
+            _ => {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    format!("usage: {}", usage),
+                ))
+            }
         }
     }
 
@@ -1097,10 +1104,12 @@ impl PrimitiveObject {
 
                 Ok(lit)
             }
-            _ => return Err(gen_error_info(
-                Position::new(interval, &data.context.flow),
-                format!("usage: {}", usage),
-            ))
+            _ => {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    format!("usage: {}", usage),
+                ))
+            }
         }
     }
 
@@ -1144,13 +1153,17 @@ impl PrimitiveObject {
                 &data.context.flow,
                 interval,
                 "".to_string(),
-            ).ok()
+            )
+            .ok()
         } else {
             None
         };
 
-
-        match (object.value.get("milliseconds"), object.value.get("timezone"), offset) {
+        match (
+            object.value.get("milliseconds"),
+            object.value.get("timezone"),
+            offset,
+        ) {
             (Some(lit), None, None) if lit.primitive.get_type() == PrimitiveType::PrimitiveInt => {
                 let millis = Literal::get_value::<i64>(
                     &lit.primitive,
@@ -1165,7 +1178,9 @@ impl PrimitiveObject {
 
                 Ok(PrimitiveString::get_literal(&formatted_date, interval))
             }
-            (Some(lit), Some(timezone), _) if lit.primitive.get_type() == PrimitiveType::PrimitiveInt => {
+            (Some(lit), Some(timezone), _)
+                if lit.primitive.get_type() == PrimitiveType::PrimitiveInt =>
+            {
                 let millis = Literal::get_value::<i64>(
                     &lit.primitive,
                     &data.context.flow,
@@ -1178,34 +1193,34 @@ impl PrimitiveObject {
                     &data.context.flow,
                     interval,
                     "".to_string(),
-                ).ok();
+                )
+                .ok();
 
                 let formatted_date = match tz_string {
                     Some(tz_string) => {
                         let local_date = Utc.timestamp_millis(*millis);
 
                         match tz_string.parse::<Tz>() {
-                            Ok(tz) => {
-                                match tz.from_local_datetime(&local_date.naive_local()) {
-                                    LocalResult::Single(date) 
-                                    | LocalResult::Ambiguous(date, _) => {
-                                        tools_time::format_date(args, date, data, interval, false)?
-                                    },
-                                    LocalResult::None => {
-                                        tools_time::format_date(args, local_date, data, interval, false)?
-                                    },
+                            Ok(tz) => match UTC.from_local_datetime(&local_date.naive_local()) {
+                                LocalResult::Single(date) | LocalResult::Ambiguous(date, _) => {
+                                    let date = date.with_timezone(&tz);
+
+                                    tools_time::format_date(args, date, data, interval, false)?
                                 }
+                                LocalResult::None => tools_time::format_date(
+                                    args, local_date, data, interval, false,
+                                )?,
                             },
                             Err(_) => {
                                 return Err(gen_error_info(
                                     Position::new(interval, &data.context.flow),
                                     format!("invalid timezone {}", tz_string),
                                 ))
-                            },
+                            }
                         }
-                    },
+                    }
                     _ => {
-                        let date =  Utc.timestamp_millis(*millis);
+                        let date = Utc.timestamp_millis(*millis);
 
                         tools_time::format_date(args, date, data, interval, false)?
                     }
@@ -1214,7 +1229,9 @@ impl PrimitiveObject {
                 Ok(PrimitiveString::get_literal(&formatted_date, interval))
             }
 
-            (Some(lit), None, Some(offset)) if lit.primitive.get_type() == PrimitiveType::PrimitiveInt => {
+            (Some(lit), None, Some(offset))
+                if lit.primitive.get_type() == PrimitiveType::PrimitiveInt =>
+            {
                 let millis = Literal::get_value::<i64>(
                     &lit.primitive,
                     &data.context.flow,
@@ -1222,19 +1239,18 @@ impl PrimitiveObject {
                     "".to_string(),
                 )?;
 
-                let date: DateTime<FixedOffset> = FixedOffset::east(*offset as i32).timestamp_millis(*millis);
+                let date: DateTime<FixedOffset> =
+                    FixedOffset::east(*offset as i32).timestamp_millis(*millis);
 
                 let formatted_date = tools_time::format_date(args, date, data, interval, false)?;
 
                 Ok(PrimitiveString::get_literal(&formatted_date, interval))
             }
 
-            _ => {
-                Err(gen_error_info(
-                    Position::new(interval, &data.context.flow),
-                    format!("usage: {}", usage),
-                ))
-            }
+            _ => Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            )),
         }
     }
 }
@@ -1844,6 +1860,7 @@ impl PrimitiveObject {
             content_type: content_type.to_owned(),
             primitive: Box::new(object.clone()),
             additional_info: None,
+            secure_variable: false,
             interval,
         })
     }
@@ -1870,16 +1887,26 @@ impl PrimitiveObject {
             ));
         }
 
-        let email_regex = Regex::new(
-            r"^([a-zA-Z0-9_+]([a-zA-Z0-9_+.]*[a-zA-Z0-9_+])?)@([a-zA-Z0-9]+([\-\.]{1}[a-zA-Z0-9]+)*\.[a-zA-Z]{2,6})",
-        )
-        .unwrap();
+        let email_regex = Regex::new(r"^[^@]+@[^@]+\.[^@]+$").unwrap();
 
         let lit = PrimitiveBoolean::get_literal(email_regex.is_match(&text), interval);
 
         Ok(lit)
     }
 
+    fn is_secure(
+        _object: &mut PrimitiveObject,
+        _args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        data: &mut Data,
+        interval: Interval,
+        _content_type: &str,
+    ) -> Result<Literal, ErrorInfo> {
+        let lit = PrimitiveBoolean::get_literal(data.event.secure, interval);
+
+        Ok(lit)
+    }
+
     fn match_args(
         object: &mut PrimitiveObject,
         args: &HashMap<String, Literal>,
@@ -2053,7 +2080,59 @@ impl PrimitiveObject {
             Some(map) if map.contains_key("error") => {
                 Ok(PrimitiveBoolean::get_literal(true, interval))
             }
-            _ => Ok(PrimitiveBoolean::get_literal(false, interval))
+            _ => Ok(PrimitiveBoolean::get_literal(false, interval)),
+        }
+    }
+
+    fn to_xml(
+        object: &mut PrimitiveObject,
+        args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        data: &mut Data,
+        interval: Interval,
+        _content_type: &str,
+    ) -> Result<Literal, ErrorInfo> {
+        let usage = "to_xml() => string";
+
+        if !args.is_empty() {
+            return Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            ));
+        }
+
+        match serde_xml_rs::to_string(&object.to_json()) {
+            Ok(string) => Ok(PrimitiveString::get_literal(&string, interval)),
+            Err(_) => Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("Object can not be converted to xml"),
+            )),
+        }
+    }
+
+    fn to_yaml(
+        object: &mut PrimitiveObject,
+        args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        data: &mut Data,
+        interval: Interval,
+        _content_type: &str,
+    ) -> Result<Literal, ErrorInfo> {
+        let usage = "to_yaml() => string";
+
+        if !args.is_empty() {
+            return Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            ));
+        }
+
+        match serde_yaml::to_string(&object.to_json()) {
+            Ok(string) => Ok(PrimitiveString::get_literal(&string, interval)),
+            Err(_) => Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("Object can not be converted to yaml"),
+            )),
         }
     }
 
@@ -2422,6 +2501,7 @@ impl PrimitiveObject {
             content_type: "object".to_owned(),
             primitive,
             additional_info: None,
+            secure_variable: false,
             interval,
         }
     }
@@ -2596,6 +2676,7 @@ impl Primitive for PrimitiveObject {
         &mut self,
         name: &str,
         args: &HashMap<String, Literal>,
+        mem_type: &MemoryType,
         additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         content_type: &ContentType,
@@ -2633,9 +2714,16 @@ impl Primitive for PrimitiveObject {
 
         for function in vector.iter() {
             if let Some((f, right)) = function.get(name) {
-                let result = f(self, args, additional_info, data, interval, &content_type)?;
+                if *mem_type == MemoryType::Constant && *right == Right::Write {
+                    return Err(gen_error_info(
+                        Position::new(interval, &data.context.flow),
+                        format!("{}", ERROR_CONSTANT_MUTABLE_FUNCTION),
+                    ));
+                } else {
+                    let result = f(self, args, additional_info, data, interval, &content_type)?;
 
-                return Ok((result, *right));
+                    return Ok((result, *right));
+                }
             }
         }
 
@@ -2646,6 +2734,7 @@ impl Primitive for PrimitiveObject {
                     return res.primitive.do_exec(
                         name,
                         args,
+                        mem_type,
                         additional_info,
                         interval,
                         &ContentType::Primitive,
