@@ -1,5 +1,4 @@
 use crate::data::error_info::ErrorInfo;
-use crate::data::literal::ContentType;
 use crate::data::position::Position;
 use crate::data::primitive::array::PrimitiveArray;
 use crate::data::primitive::boolean::PrimitiveBoolean;
@@ -10,7 +9,8 @@ use crate::data::primitive::object::PrimitiveObject;
 use crate::data::primitive::tools::*;
 use crate::data::primitive::Right;
 use crate::data::primitive::{Primitive, PrimitiveType};
-use crate::data::{ast::Interval, message::Message, Data, Literal, MessageData, MSG};
+use crate::data::{ast::Interval, message::Message, Data, Literal, MemoryType, MessageData, MSG};
+use crate::data::{literal, literal::ContentType};
 use crate::error_format::*;
 use crate::interpreter::json_to_literal;
 use phf::phf_map;
@@ -26,6 +26,7 @@ use std::{collections::HashMap, sync::mpsc};
 type PrimitiveMethod = fn(
     string: &mut PrimitiveString,
     args: &HashMap<String, Literal>,
+    additional_info: &Option<HashMap<String, Literal>>,
     interval: Interval,
     data: &mut Data,
     msg_data: &mut MessageData,
@@ -37,7 +38,10 @@ const FUNCTIONS: phf::Map<&'static str, (PrimitiveMethod, Right)> = phf_map! {
     "is_int" => (PrimitiveString::is_int as PrimitiveMethod, Right::Read),
     "is_float" => (PrimitiveString::is_float as PrimitiveMethod, Right::Read),
     "type_of" => (PrimitiveString::type_of as PrimitiveMethod, Right::Read),
+    "get_info" => (PrimitiveString::get_info as PrimitiveMethod, Right::Read),
+    "is_error" => (PrimitiveString::is_error as PrimitiveMethod, Right::Read),
     "to_string" => (PrimitiveString::to_string as PrimitiveMethod, Right::Read),
+    "to_json" => (PrimitiveString::to_csml_json as PrimitiveMethod, Right::Read),
 
     "is_email" => (PrimitiveString::is_email as PrimitiveMethod, Right::Read),
     "append" => (PrimitiveString::append as PrimitiveMethod, Right::Read),
@@ -62,6 +66,10 @@ const FUNCTIONS: phf::Map<&'static str, (PrimitiveMethod, Right)> = phf_map! {
     "slice" => (PrimitiveString::slice as PrimitiveMethod, Right::Read),
     "split" => (PrimitiveString::split as PrimitiveMethod, Right::Read),
 
+    "trim" => (PrimitiveString::trim as PrimitiveMethod, Right::Read),
+    "trim_left" => (PrimitiveString::trim_left as PrimitiveMethod, Right::Read),
+    "trim_right" => (PrimitiveString::trim_right as PrimitiveMethod, Right::Read),
+
     "abs" => (PrimitiveString::abs as PrimitiveMethod, Right::Read),
     "cos" => (PrimitiveString::cos as PrimitiveMethod, Right::Read),
     "ceil" =>(PrimitiveString::ceil as PrimitiveMethod, Right::Read),
@@ -88,6 +96,7 @@ impl PrimitiveString {
     fn is_number(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -110,6 +119,7 @@ impl PrimitiveString {
     fn is_int(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -132,6 +142,7 @@ impl PrimitiveString {
     fn is_float(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -159,6 +170,7 @@ impl PrimitiveString {
     fn is_email(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -173,10 +185,7 @@ impl PrimitiveString {
             ));
         }
 
-        let email_regex = Regex::new(
-            r"^([a-zA-Z0-9_+]([a-zA-Z0-9_+.]*[a-zA-Z0-9_+])?)@([a-zA-Z0-9]+([\-\.]{1}[a-zA-Z0-9]+)*\.[a-zA-Z]{2,6})",
-        )
-        .unwrap();
+        let email_regex = Regex::new(r"^[^@]+@[^@]+\.[^@]+$").unwrap();
 
         let result = email_regex.is_match(&string.value);
 
@@ -186,6 +195,7 @@ impl PrimitiveString {
     fn type_of(
         _string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -203,9 +213,39 @@ impl PrimitiveString {
         Ok(PrimitiveString::get_literal("string", interval))
     }
 
+    fn get_info(
+        _string: &mut PrimitiveString,
+        args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        literal::get_info(args, additional_info, interval, data)
+    }
+
+    fn is_error(
+        _string: &mut PrimitiveString,
+        _args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        _data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        match additional_info {
+            Some(map) if map.contains_key("error") => {
+                Ok(PrimitiveBoolean::get_literal(true, interval))
+            }
+            _ => Ok(PrimitiveBoolean::get_literal(false, interval)),
+        }
+    }
+
     fn to_string(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -222,12 +262,49 @@ impl PrimitiveString {
 
         Ok(PrimitiveString::get_literal(&string.to_string(), interval))
     }
+
+    fn to_csml_json(
+        string: &mut PrimitiveString,
+        args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        let usage = "to_json() => obj";
+
+        if !args.is_empty() {
+            return Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            ));
+        }
+
+        let value = string.to_string();
+
+        let xml: Option<serde_json::Value> = serde_xml_rs::from_str(&value).ok();
+        let yaml: Option<serde_json::Value> = serde_yaml::from_str(&value).ok();
+
+        match (&yaml, &xml) {
+            (_, Some(json)) | (Some(json), _) => {
+                json_to_literal(json, interval, &data.context.flow)
+            }
+            _ => {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    format!("Invalid format string is not a valid yaml or xml"),
+                ));
+            }
+        }
+    }
 }
 
 impl PrimitiveString {
     fn append(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -269,6 +346,7 @@ impl PrimitiveString {
     fn contains(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -308,6 +386,7 @@ impl PrimitiveString {
     fn contains_regex(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -357,6 +436,7 @@ impl PrimitiveString {
     fn replace(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -372,23 +452,24 @@ impl PrimitiveString {
         }
 
         let (to_replace, replace_by) = match (args.get("arg0"), args.get("arg1")) {
-            (Some(old), Some(new)) 
-                if old.primitive.get_type() == PrimitiveType::PrimitiveString &&
-                new.primitive.get_type() == PrimitiveType::PrimitiveString 
-            => {
-
-                (Literal::get_value::<String>(
-                    &old.primitive,
-                    &data.context.flow,
-                    interval,
-                    ERROR_STRING_REPLACE.to_owned(),
-                )?,
-                Literal::get_value::<String>(
-                    &new.primitive,
-                    &data.context.flow,
-                    interval,
-                    ERROR_STRING_REPLACE.to_owned(),
-                )?)
+            (Some(old), Some(new))
+                if old.primitive.get_type() == PrimitiveType::PrimitiveString
+                    && new.primitive.get_type() == PrimitiveType::PrimitiveString =>
+            {
+                (
+                    Literal::get_value::<String>(
+                        &old.primitive,
+                        &data.context.flow,
+                        interval,
+                        ERROR_STRING_REPLACE.to_owned(),
+                    )?,
+                    Literal::get_value::<String>(
+                        &new.primitive,
+                        &data.context.flow,
+                        interval,
+                        ERROR_STRING_REPLACE.to_owned(),
+                    )?,
+                )
             }
             _ => {
                 return Err(gen_error_info(
@@ -406,6 +487,7 @@ impl PrimitiveString {
     fn replace_all(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -421,23 +503,24 @@ impl PrimitiveString {
         }
 
         let (to_replace, replace_by) = match (args.get("arg0"), args.get("arg1")) {
-            (Some(old), Some(new)) 
-                if old.primitive.get_type() == PrimitiveType::PrimitiveString &&
-                new.primitive.get_type() == PrimitiveType::PrimitiveString 
-            => {
-
-                (Literal::get_value::<String>(
-                    &old.primitive,
-                    &data.context.flow,
-                    interval,
-                    ERROR_STRING_REPLACE_ALL.to_owned(),
-                )?,
-                Literal::get_value::<String>(
-                    &new.primitive,
-                    &data.context.flow,
-                    interval,
-                    ERROR_STRING_REPLACE_ALL.to_owned(),
-                )?)
+            (Some(old), Some(new))
+                if old.primitive.get_type() == PrimitiveType::PrimitiveString
+                    && new.primitive.get_type() == PrimitiveType::PrimitiveString =>
+            {
+                (
+                    Literal::get_value::<String>(
+                        &old.primitive,
+                        &data.context.flow,
+                        interval,
+                        ERROR_STRING_REPLACE_ALL.to_owned(),
+                    )?,
+                    Literal::get_value::<String>(
+                        &new.primitive,
+                        &data.context.flow,
+                        interval,
+                        ERROR_STRING_REPLACE_ALL.to_owned(),
+                    )?,
+                )
             }
             _ => {
                 return Err(gen_error_info(
@@ -455,6 +538,7 @@ impl PrimitiveString {
     fn replace_regex(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -470,23 +554,24 @@ impl PrimitiveString {
         }
 
         let (regex, replace_by) = match (args.get("arg0"), args.get("arg1")) {
-            (Some(old), Some(new)) 
-                if old.primitive.get_type() == PrimitiveType::PrimitiveString &&
-                new.primitive.get_type() == PrimitiveType::PrimitiveString 
-            => {
-
-                (Literal::get_value::<String>(
-                    &old.primitive,
-                    &data.context.flow,
-                    interval,
-                    ERROR_STRING_REPLACE_REGEX.to_owned(),
-                )?,
-                Literal::get_value::<String>(
-                    &new.primitive,
-                    &data.context.flow,
-                    interval,
-                    ERROR_STRING_REPLACE_REGEX.to_owned(),
-                )?)
+            (Some(old), Some(new))
+                if old.primitive.get_type() == PrimitiveType::PrimitiveString
+                    && new.primitive.get_type() == PrimitiveType::PrimitiveString =>
+            {
+                (
+                    Literal::get_value::<String>(
+                        &old.primitive,
+                        &data.context.flow,
+                        interval,
+                        ERROR_STRING_REPLACE_REGEX.to_owned(),
+                    )?,
+                    Literal::get_value::<String>(
+                        &new.primitive,
+                        &data.context.flow,
+                        interval,
+                        ERROR_STRING_REPLACE_REGEX.to_owned(),
+                    )?,
+                )
             }
             _ => {
                 return Err(gen_error_info(
@@ -514,6 +599,7 @@ impl PrimitiveString {
     fn ends_with(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -553,6 +639,7 @@ impl PrimitiveString {
     fn ends_with_regex(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -606,6 +693,7 @@ impl PrimitiveString {
     fn from_json(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -636,6 +724,7 @@ impl PrimitiveString {
     fn is_empty(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -658,6 +747,7 @@ impl PrimitiveString {
     fn length(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -680,6 +770,7 @@ impl PrimitiveString {
     fn do_match(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -727,6 +818,7 @@ impl PrimitiveString {
     fn do_match_regex(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -789,6 +881,7 @@ impl PrimitiveString {
     fn starts_with(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -828,6 +921,7 @@ impl PrimitiveString {
     fn starts_with_regex(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -881,6 +975,7 @@ impl PrimitiveString {
     fn to_lowercase(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -902,6 +997,7 @@ impl PrimitiveString {
     fn to_uppercase(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -923,6 +1019,7 @@ impl PrimitiveString {
     fn capitalize(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -951,6 +1048,7 @@ impl PrimitiveString {
     fn slice(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -1060,6 +1158,7 @@ impl PrimitiveString {
     fn split(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -1099,12 +1198,80 @@ impl PrimitiveString {
 
         Ok(PrimitiveArray::get_literal(&vector, interval))
     }
+
+    fn trim(
+        string: &mut PrimitiveString,
+        args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        let usage = "trim() => string";
+
+        if !args.is_empty() {
+            return Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            ));
+        }
+
+        let s = &string.value;
+        Ok(PrimitiveString::get_literal(&s.trim(), interval))
+    }
+
+    fn trim_left(
+        string: &mut PrimitiveString,
+        args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        let usage = "trim_left() => string";
+
+        if !args.is_empty() {
+            return Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            ));
+        }
+
+        let s = &string.value;
+        Ok(PrimitiveString::get_literal(&s.trim_start(), interval))
+    }
+
+    fn trim_right(
+        string: &mut PrimitiveString,
+        args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        let usage = "trim_right() => string";
+
+        if !args.is_empty() {
+            return Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            ));
+        }
+
+        let s = &string.value;
+        Ok(PrimitiveString::get_literal(&s.trim_end(), interval))
+    }
 }
 
+// memory type can be set tu 'use' because the result of the operation will create a new literal.
 impl PrimitiveString {
     fn abs(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -1116,6 +1283,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "abs",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1131,6 +1300,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "abs",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1150,6 +1321,7 @@ impl PrimitiveString {
     fn cos(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -1161,6 +1333,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "cos",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1177,6 +1351,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "cos",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1196,6 +1372,7 @@ impl PrimitiveString {
     fn ceil(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -1207,6 +1384,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "ceil",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1222,6 +1401,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "ceil",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1241,6 +1422,7 @@ impl PrimitiveString {
     fn pow(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -1252,6 +1434,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "pow",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1267,6 +1451,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "pow",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1286,6 +1472,7 @@ impl PrimitiveString {
     fn floor(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -1297,6 +1484,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "floor",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1312,6 +1501,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "floor",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1331,6 +1522,7 @@ impl PrimitiveString {
     fn round(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -1342,6 +1534,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "round",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1357,6 +1551,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "round",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1376,6 +1572,7 @@ impl PrimitiveString {
     fn sin(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -1387,6 +1584,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "sin",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1402,6 +1601,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "sin",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1421,6 +1622,7 @@ impl PrimitiveString {
     fn sqrt(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -1432,6 +1634,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "sqrt",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1447,6 +1651,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "sqrt",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1466,6 +1672,7 @@ impl PrimitiveString {
     fn tan(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -1477,6 +1684,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "tan",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1492,6 +1701,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "tan",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1511,6 +1722,7 @@ impl PrimitiveString {
     fn to_int(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -1522,6 +1734,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "to_int",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1537,6 +1751,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "to_int",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1556,6 +1772,7 @@ impl PrimitiveString {
     fn to_float(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -1567,6 +1784,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "to_float",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1582,6 +1801,8 @@ impl PrimitiveString {
             let (literal, _right) = primitive.do_exec(
                 "to_float",
                 args,
+                &MemoryType::Use,
+                additional_info,
                 interval,
                 &ContentType::Primitive,
                 data,
@@ -1616,6 +1837,8 @@ impl PrimitiveString {
         Literal {
             content_type: "string".to_owned(),
             primitive,
+            additional_info: None,
+            secure_variable: false,
             interval,
         }
     }
@@ -1688,8 +1911,7 @@ impl Primitive for PrimitiveString {
                     other.get_type()
                 )),
             }
-        }
-        else {
+        } else {
             let mut new_string = self.value.clone();
 
             new_string.push_str(&rhs.value);
@@ -1874,6 +2096,8 @@ impl Primitive for PrimitiveString {
             Literal {
                 content_type: "string".to_owned(),
                 primitive: Box::new(PrimitiveString::new(&self.value)),
+                additional_info: None,
+                secure_variable: false,
                 interval: Interval {
                     start_column: 0,
                     start_line: 0,
@@ -1906,6 +2130,8 @@ impl Primitive for PrimitiveString {
         &mut self,
         name: &str,
         args: &HashMap<String, Literal>,
+        mem_type: &MemoryType,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         _content_type: &ContentType,
         data: &mut Data,
@@ -1913,9 +2139,24 @@ impl Primitive for PrimitiveString {
         sender: &Option<mpsc::Sender<MSG>>,
     ) -> Result<(Literal, Right), ErrorInfo> {
         if let Some((f, right)) = FUNCTIONS.get(name) {
-            let res = f(self, args, interval, data, msg_data, sender)?;
+            if *mem_type == MemoryType::Constant && *right == Right::Write {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    format!("{}" , ERROR_CONSTANT_MUTABLE_FUNCTION),
+                ));
+            } else {
+                let res = f(
+                    self,
+                    args,
+                    additional_info,
+                    interval,
+                    data,
+                    msg_data,
+                    sender,
+                )?;
 
-            return Ok((res, *right));
+                return Ok((res, *right));
+            }
         }
 
         Err(gen_error_info(
