@@ -1,9 +1,9 @@
 pub mod data;
 
 use crate::data::tokens::Span;
-use crate::data::{position::Position, Interval, warnings::Warnings};
+use crate::data::{position::Position, warnings::Warnings, Interval};
 use nom::{
-    error::{ErrorKind, ParseError},
+    error::{ContextError, ErrorKind, ParseError},
     *,
 };
 
@@ -66,7 +66,8 @@ pub const ERROR_FUNCTIONS_ARGS: &str = "function arguments must be in an array";
 pub const ERROR_EXPR_TO_LITERAL: &str = "expression can't be converted to Literal";
 pub const ERROR_PAYLOAD_EXCEED_MAX_SIZE: &str = "payload exceeds max payload size (16kb)";
 
-pub const ERROR_STEP_LIMIT: &str = "[Infinite loop] Step limit reached: 100 steps where executed in a single run";
+pub const ERROR_STEP_LIMIT: &str =
+    "[Infinite loop] Step limit reached: 100 steps where executed in a single run";
 
 // Event
 pub const ERROR_EVENT_CONTENT_TYPE: &str = "event can only be of ContentType::Event";
@@ -80,7 +81,7 @@ pub const ERROR_COMPONENT_UNKNOWN: &str = "function does not exist for component
 
 // Fn API
 pub const ERROR_FN_ID: &str = "App name must be of type string";
-pub const ERROR_FN_ENDPOINT: &str = "App can not be called because fn_endpoint is not set in bot";
+pub const ERROR_FN_ENDPOINT: &str = "App can not be called because apps_endpoint is not set in bot";
 pub const ERROR_FAIL_RESPONSE_JSON: &str = "failed to read response as JSON";
 
 // ### Import
@@ -89,7 +90,7 @@ pub const ERROR_IMPORT_STEP_FLOW: &str = "step not found in flow";
 
 // ### Variables
 pub const ERROR_GET_VAR_INFO: &str = "Expression must be a variable";
-pub const ERROR_JSON_TO_LITERAL: &str = "Number is larget than a 64-bit integer";
+pub const ERROR_JSON_TO_LITERAL: &str = "Number is larger than a 64-bit integer";
 
 // ### Memory
 pub const ERROR_STEP_MEMORY: &str = "Variable does not exist in step's memory";
@@ -116,6 +117,8 @@ pub const ERROR_CAROUSEL: &str =
     "argument 'cards' in Carousel component must be of type Array<Card>";
 pub const ERROR_ONE_OF: &str =
     "OneOf builtin expects one value of type Array. Example: OneOf( [1, 2, 3] )";
+pub const ERROR_VAR_EXISTS: &str =
+    "Exists builtin expects one value of type String. Example: Exists( \"var_name\" )";
 pub const ERROR_SHUFFLE: &str =
     "Shuffle builtin expects one value of type Array. Example: Shuffle( [1, 2, 3] )";
 pub const ERROR_LENGTH: &str =
@@ -134,18 +137,36 @@ pub const ERROR_AUDIO: &str =
     "Audio component expects one argument of type string. Example: Audio(url = \"hola\")";
 pub const ERROR_FILE: &str =
     "File component expects one argument of type string. Example: File(url = \"hola\")";
-pub const ERROR_HTTP: &str =
-    "HTTP builtin expects one url of type string. Example: HTTP(\"https://clevy.io\")";
 pub const ERROR_HTTP_GET_VALUE: &str =
-"not found in HTTP object. Use the HTTP() builtin to construct the correct object to make HTTP calls";
+    "not found in HTTP object. Use the HTTP() builtin to construct the correct object to make HTTP calls";
 pub const ERROR_HTTP_QUERY_VALUES: &str =
     "must have a value of type String. Example: {key: \"value\"}";
+pub const ERROR_HTTP: &str =
+    "HTTP builtin expects one url of type string. Example: HTTP(\"https://clevy.io\")";
+pub const ERROR_JWT: &str = "JWT builtin expects payload as argument. Example: JWT({
+        \"user\": \"name\",
+        \"somekey\": {
+          \"somevalue\": 42
+        },
+        \"exp\": 1618064023,
+        \"iss\": \"CSML STUDIO\"
+      })";
+pub const ERROR_SMTP: &str =
+    "SMTP builtin expects SMTP Server Address. Example: SMTP(\"smtp.gmail.com\")";
+pub const ERROR_CRYPTO: &str =
+    "CRYPTO builtin expects one argument of type string. Example: CRYPTO(\"text\")";
 pub const ERROR_BUILTIN_UNKNOWN: &str = "Unknown builtin";
 
 // ### native Components
 pub const ERROR_HTTP_NOT_DATA: &str = "bad format: no 'data' in HTTP response";
 pub const ERROR_NATIVE_COMPONENT: &str = "native component does not exist";
 
+// ### Constants
+pub const ERROR_CONSTANT_MUTABLE_FUNCTION: &str =
+    "Invalid operation constants can not execute self mutable functions";
+pub const ERROR_INVALID_CONSTANT_EXPR: &str =
+    "Constant invalid expression type: constants can not be assign this type of expression";
+
 // ### Primitives
 // #### Indexing
 pub const ERROR_INDEXING: &str =
@@ -177,6 +198,12 @@ pub const ERROR_STRING_APPEND: &str =
     "[append] takes one parameter of type String. Usage: string.append(\"text to append\")";
 pub const ERROR_STRING_CONTAINS: &str =
     "[contains] takes one parameter of type String. Usage: string.contains(\"word\")";
+pub const ERROR_STRING_REPLACE: &str =
+    "[replace] takes tow parameter of type String. Usage: \"this is old\".replace(\"old\", \"new\")";
+pub const ERROR_STRING_REPLACE_ALL: &str =
+    "[replace_all] takes tow parameter of type String. Usage: \"old old old old\".replace_all(\"old\", \"new\")";
+pub const ERROR_STRING_REPLACE_REGEX: &str =
+    "[replace_regex] takes tow parameter of type String. Usage: \"hello world\".replace_regex(\"world\", \"Clevy\")";
 pub const ERROR_STRING_CONTAINS_REGEX: &str =
     "[contains_regex] takes one parameter of type String. Usage: string.contains_regex(\"regex\")";
 pub const ERROR_STRING_VALID_REGEX: &str = "parameter must be a valid regex expression"; // link to docs
@@ -356,15 +383,12 @@ pub fn gen_error_info(position: Position, message: String) -> ErrorInfo {
 }
 
 pub fn gen_warning_info(position: Position, message: String) -> Warnings {
-    Warnings {
-        position,
-        message
-    }
+    Warnings { position, message }
 }
 
 pub fn gen_nom_error<'a, E>(span: Span<'a>, error: &'static str) -> Err<E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     Err::Error(E::add_context(
         span,
@@ -375,7 +399,7 @@ where
 
 pub fn gen_nom_failure<'a, E>(span: Span<'a>, error: &'static str) -> Err<E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     Err::Failure(E::add_context(
         span,
@@ -410,8 +434,10 @@ pub fn convert_error_from_interval<'a>(
 }
 
 pub fn gen_infinite_loop_error_msg(infinite_loop: Vec<(String, String)>) -> String {
-    infinite_loop.iter().fold( String::new(), |mut acc, (flow, step)| {
-        acc.push_str(&format!("[flow] {}, [step] {}\n", flow, step));
-        acc
-    })
+    infinite_loop
+        .iter()
+        .fold(String::new(), |mut acc, (flow, step)| {
+            acc.push_str(&format!("[flow] {}, [step] {}\n", flow, step));
+            acc
+        })
 }
