@@ -1,17 +1,13 @@
 use crate::data::{
-    ast::{Expr},
-    tokens::{Span, WHILE, L_PAREN, R_PAREN},
+    ast::Expr,
+    tokens::{Span, L_PAREN, R_PAREN, WHILE},
 };
 use crate::parser::operator::parse_operator;
-use crate::parser::{
-    parse_comments::comment,
-    parse_scope::parse_scope,
-    tools::{get_interval},
-};
+use crate::parser::{parse_comments::comment, parse_scope::parse_scope, tools::get_interval};
 use nom::{
     bytes::complete::tag,
-    combinator::{cut},
-    error::ParseError,
+    combinator::cut,
+    error::{ContextError, ParseError},
     sequence::preceded,
     *,
 };
@@ -22,7 +18,7 @@ use nom::{
 
 pub fn parse_while<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, _) = preceded(comment, tag(WHILE))(s)?;
     let (s, mut interval) = get_interval(s)?;
@@ -35,8 +31,5 @@ where
     let (s, end) = get_interval(s)?;
     interval.add_end(end);
 
-    Ok((
-        s,
-        Expr::WhileExpr(Box::new(expr), block, interval),
-    ))
+    Ok((s, Expr::WhileExpr(Box::new(expr), block, interval)))
 }
